"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContainerInstance = void 0;
const container_class_1 = require("./container.class");
const missing_provided_service_type_error_1 = require("./error/missing-provided-service-type.error");
const service_not_found_error_1 = require("./error/service-not-found.error");
const token_class_1 = require("./token.class");
const empty_const_1 = require("./empty.const");
/**
 * TypeDI can have multiple containers.
 * One container is ContainerInstance.
 */
class ContainerInstance {
    constructor(id) {
        /** All registered services in the container. */
        this.services = [];
        this.id = id;
    }
    has(identifier) {
        return !!this.findService(identifier);
    }
    get(identifier) {
        const globalContainer = container_class_1.Container.of(undefined);
        const globalService = globalContainer.findService(identifier);
        const scopedService = this.findService(identifier);
        if (globalService && globalService.global === true)
            return this.getServiceValue(globalService);
        if (scopedService)
            return this.getServiceValue(scopedService);
        /** If it's the first time requested in the child container we load it from parent and set it. */
        if (globalService && this !== globalContainer) {
            const clonedService = { ...globalService };
            clonedService.value = empty_const_1.EMPTY_VALUE;
            /**
             * We need to immediately set the empty value from the root container
             * to prevent infinite lookup in cyclic dependencies.
             */
            this.set(clonedService);
            const value = this.getServiceValue(clonedService);
            this.set({ ...clonedService, value });
            return value;
        }
        if (globalService)
            return this.getServiceValue(globalService);
        throw new service_not_found_error_1.ServiceNotFoundError(identifier);
    }
    getMany(id) {
        return this.filterServices(id).map(service => this.getServiceValue(service));
    }
    set(identifierOrServiceMetadata, value) {
        if (identifierOrServiceMetadata instanceof Array) {
            identifierOrServiceMetadata.forEach(data => this.set(data));
            return this;
        }
        if (typeof identifierOrServiceMetadata === 'string' || identifierOrServiceMetadata instanceof token_class_1.Token) {
            return this.set({
                id: identifierOrServiceMetadata,
                type: null,
                value: value,
                factory: undefined,
                global: false,
                multiple: false,
                transient: false,
            });
        }
        if (typeof identifierOrServiceMetadata === 'function') {
            return this.set({
                id: identifierOrServiceMetadata,
                // TODO: remove explicit casting
                type: identifierOrServiceMetadata,
                value: value,
                factory: undefined,
                global: false,
                multiple: false,
                transient: false,
            });
        }
        const newService = {
            id: new token_class_1.Token('UNREACHABLE'),
            type: null,
            factory: undefined,
            value: empty_const_1.EMPTY_VALUE,
            global: false,
            multiple: false,
            transient: false,
            ...identifierOrServiceMetadata,
        };
        const service = this.findService(newService.id);
        if (service && service.multiple !== true) {
            Object.assign(service, newService);
        }
        else {
            this.services.push(newService);
        }
        return this;
    }
    /**
     * Removes services with a given service identifiers (tokens or types).
     */
    remove(...ids) {
        ids.forEach(id => {
            this.filterServices(id).forEach(service => {
                this.services.splice(this.services.indexOf(service), 1);
            });
        });
        return this;
    }
    /**
     * Completely resets the container by removing all previously registered services from it.
     */
    reset(options = { strategy: 'resetValue' }) {
        // TODO: The resources should be freed up.
        switch (options.strategy) {
            case 'resetValue':
                this.services = this.services.map(s => {
                    return {
                        ...s,
                        value: !!s.type ? empty_const_1.EMPTY_VALUE : s.value,
                    };
                });
                break;
            case 'resetValue':
                this.services = [];
                break;
            default:
                throw new Error('Received invalid reset strategy.');
        }
        return this;
    }
    /**
     * Filters registered service in the with a given service identifier.
     */
    filterServices(identifier) {
        return this.services.filter(service => {
            if (service.id)
                return service.id === identifier;
            if (service.type && typeof identifier === 'function')
                return service.type === identifier || identifier.prototype instanceof service.type;
            return false;
        });
    }
    /**
     * Finds registered service in the with a given service identifier.
     */
    findService(identifier) {
        return this.services.find(service => {
            if (service.id) {
                if (identifier instanceof Object &&
                    service.id instanceof token_class_1.Token &&
                    identifier.service instanceof token_class_1.Token) {
                    return service.id === identifier.service;
                }
                return service.id === identifier;
            }
            // todo: not sure why it was here || identifier.prototype instanceof service.type;
            if (service.type && typeof identifier === 'function') {
                return service.type === identifier;
            }
            return false;
        });
    }
    /**
     * Gets service value.
     */
    getServiceValue(serviceMetadata) {
        // find if instance of this object already initialized in the container and return it if it is
        /**
         * If the service value has been set to anything prior to this call we return the set value.
         * NOTE: This part builds on the assumption that transient dependencies has no value set ever.
         */
        if (serviceMetadata.value !== empty_const_1.EMPTY_VALUE) {
            return serviceMetadata.value;
        }
        // if named service was requested and its instance was not found plus there is not type to know what to initialize,
        // this means service was not pre-registered and we throw an exception
        if ((!serviceMetadata || !serviceMetadata.type) &&
            (!serviceMetadata || !serviceMetadata.factory) &&
            (typeof serviceMetadata.id === 'string' || serviceMetadata.id instanceof token_class_1.Token))
            throw new service_not_found_error_1.ServiceNotFoundError(serviceMetadata.id);
        // at this point we either have type in service registered, either identifier is a target type
        let construtableType;
        if (serviceMetadata && serviceMetadata.type) {
            construtableType = serviceMetadata.type;
        }
        else if (typeof serviceMetadata.id === 'function') {
            construtableType = serviceMetadata.id;
        }
        // setup constructor parameters for a newly initialized service
        const paramTypes = construtableType && Reflect && Reflect.getMetadata
            ? Reflect.getMetadata('design:paramtypes', construtableType)
            : undefined;
        // TODO: remove as any, here it cannot be undefined but TS doesn't see this.
        let params = paramTypes ? this.initializeParams(construtableType, paramTypes) : [];
        // if factory is set then use it to create service instance
        let value;
        if (serviceMetadata.factory) {
            // filter out non-service parameters from created service constructor
            // non-service parameters can be, lets say Car(name: string, isNew: boolean, engine: Engine)
            // where name and isNew are non-service parameters and engine is a service parameter
            params = params.filter(param => param !== undefined);
            if (serviceMetadata.factory instanceof Array) {
                // use special [Type, "create"] syntax to allow factory services
                // in this case Type instance will be obtained from Container and its method "create" will be called
                let factoryInstance;
                try {
                    factoryInstance = this.get(serviceMetadata.factory[0]);
                }
                catch (error) {
                    if (error instanceof service_not_found_error_1.ServiceNotFoundError) {
                        factoryInstance = new serviceMetadata.factory[0]();
                    }
                }
                // TODO: rethink this
                // We need to pass `this` to make the following test pass:
                // "should support function injection with Token dependencies"
                // We should have the dependencies here, instead of passing the container to the factory.
                // Maybe we should save the dependency ID list on the meta-data?
                value = factoryInstance[serviceMetadata.factory[1]](...params, this);
            }
            else {
                // regular factory function
                value = serviceMetadata.factory(...params, this);
            }
        }
        else {
            // TODO: Commented it out as this makes no sense.
            // params.unshift(null);
            // "extra feature" - always pass container instance as the last argument to the service function
            // this allows us to support javascript where we don't have decorators and emitted metadata about dependencies
            // need to be injected, and user can use provided container to get instances he needs
            params.push(this);
            if (!construtableType)
                throw new missing_provided_service_type_error_1.MissingProvidedServiceTypeError(serviceMetadata.id);
            // eslint-disable-next-line prefer-spread
            value = new construtableType(...params);
        }
        if (serviceMetadata && !serviceMetadata.transient && value)
            serviceMetadata.value = value;
        if (construtableType)
            this.applyPropertyHandlers(construtableType, value);
        return value;
    }
    /**
     * Initializes all parameter types for a given target service class.
     */
    initializeParams(target, paramTypes) {
        return paramTypes.map((paramType, index) => {
            const paramHandler = container_class_1.Container.handlers.find(handler => {
                /**
                 * @Inject()-ed values are stored as parameter handlers and they reference their target
                 * when created. So when a class is extended the @Inject()-ed values are not inherited
                 * because the handler still points to the old object only.
                 *
                 * As a quick fix a single level parent lookup is added via `Object.getPrototypeOf(target)`,
                 * however this should be updated to a more robust solution.
                 *
                 * TODO: Add proper inheritance handling: either copy the handlers when a class is registered what
                 * TODO: has it's parent already registered as dependency or make the lookup search up to the base Object.
                 */
                return ((handler.object === target || handler.object === Object.getPrototypeOf(target)) && handler.index === index);
            });
            if (paramHandler)
                return paramHandler.value(this);
            if (paramType && paramType.name && !this.isTypePrimitive(paramType.name)) {
                return this.get(paramType);
            }
            return undefined;
        });
    }
    /**
     * Checks if given type is primitive (e.g. string, boolean, number, object).
     */
    isTypePrimitive(param) {
        return ['string', 'boolean', 'number', 'object'].indexOf(param.toLowerCase()) !== -1;
    }
    /**
     * Applies all registered handlers on a given target class.
     */
    applyPropertyHandlers(target, instance) {
        container_class_1.Container.handlers.forEach(handler => {
            if (typeof handler.index === 'number')
                return;
            if (handler.object.constructor !== target && !(target.prototype instanceof handler.object.constructor))
                return;
            if (handler.propertyName) {
                instance[handler.propertyName] = handler.value(this);
            }
        });
    }
}
exports.ContainerInstance = ContainerInstance;
//# sourceMappingURL=container-instance.class.js.map